#include <Wire.h>
#include <Servo.h> 

#define SERVO_PIN 9

//Sensor pin should be the interrupt number unless on a Due
#define SENSOR_PIN 0

int state = 0;
int moduleStatus = 0;

bool partDispensed = true;

/*
Dispenser is a state machine, the value of the state governs what 'register' the i2c bus is reading from or writing to.
0x00 = Status, 0x00 for all is OK (used for autodiscovery for now)
0x01 = Next 4 reads will be bytes comprising the 32 bit product ID being dispensed
0x02 = Next 4 reads will be bytes comprising the 32 bit stock level

0x10  = Next received byte is a quantity to dispense
*/

int address = 12;
int product_id = 451;
int product_qty = 45;

byte product_id_8bit[4];

int byteSent = 0;

Servo rotorServo;
int setPosition = 0;

int rotorHome = 30;
int rotorDispense = 120;

int rotorDuration = 500;

// Timer variables
int lastTick = 0;

void setup()
{
  Wire.begin(address);                // join i2c bus with address #4
  Wire.onReceive(receiveEvent); // register event
  Wire.onRequest(sendResponse);
  Serial.begin(9600);           // start serial for output
  rotorServo.attach(SERVO_PIN);
  rotorServo.write(rotorHome);
  splitints(); //Converts the 32 bit integers we need to communicate into arrays of 4 bytes for easy sending.
  
  attachInterrupt(SENSOR_PIN, confirm_dispense, FALLING);
}

void loop()
{
  delay(1);
}

void dispense(int qty)
{
  moduleStatus = 1;
  for (int i = 1; i<= qty; i++)
  {
    rotorServo.write(rotorDispense);
    delay(rotorDuration);
    rotorServo.write(rotorHome);
    delay(rotorDuration);
    while (!partDispensed) { delay(1); } //Wait for the interrupt from the sensor pin, make this not horribly blocking later.
  }
  moduleStatus = 0;
}

void confirm_dispense()
{
  if (!partDispensed) { partDispensed = true; }
}

void receiveEvent(int howMany)
{
  int x = Wire.read();    // receive byte as an integer
  if ( state == 10 ) {
    dispense(x);
    state = 0;
  } else {
    Serial.print("Setting state to");
    Serial.println(x);
    state = x;
    byteSent = 0;
  }
}

void sendResponse()
{
  // Send the status code (0x00 = ready, 0x01 = dispensing, > 0x02 = error)
  if ( state == 0 ) { Wire.write(moduleStatus); }
  
  // Send the stocked product code
  if ( state == 1 ) { Wire.write(product_id_8bit[byteSent]); byteSent++; }
}

void splitints()
{
  product_id_8bit[0]=(product_id >>24) & 0xff;
  product_id_8bit[1]=(product_id >>16) & 0xff;
  product_id_8bit[2]=(product_id >>8) & 0xff;
  product_id_8bit[3]=product_id & 0xff;
}
